@@ -36,16 +36,16 @@ class Lattice(object):
   The lattice class is normally used only for the geometry-lattice variable, and specifies the three lattice directions of the crystal and the lengths of the corresponding lattice vectors.
 
   lattice
-      Properties: 
+      Properties:
 
   basis1, basis2, basis3 [vector3]
-      The three lattice directions of the crystal, specified in the cartesian basis. The lengths of these vectors are ignored--only their directions matter. The lengths are determined by the basis-size property, below. These vectors are then used as a basis for all other 3-vectors in the ctl file. They default to the x, y, and z directions, respectively. 
+      The three lattice directions of the crystal, specified in the cartesian basis. The lengths of these vectors are ignored--only their directions matter. The lengths are determined by the basis-size property, below. These vectors are then used as a basis for all other 3-vectors in the ctl file. They default to the x, y, and z directions, respectively.
 
   basis-size [vector3]
-      The components of basis-size are the lengths of the three basis vectors, respectively. They default to unit lengths. 
+      The components of basis-size are the lengths of the three basis vectors, respectively. They default to unit lengths.
 
   size [vector3]
-      The size of the lattice (i.e. the length of the lattice vectors Ri, in which the crystal is periodic) in units of the basis vectors. Thus, the actual lengths of the lattice vectors are given by the components of size multiplied by the components of basis-size. (Alternatively, you can think of size as the vector between opposite corners of the primitive cell, specified in the lattice basis.) Defaults to unit lengths. 
+      The size of the lattice (i.e. the length of the lattice vectors Ri, in which the crystal is periodic) in units of the basis vectors. Thus, the actual lengths of the lattice vectors are given by the components of size multiplied by the components of basis-size. (Alternatively, you can think of size as the vector between opposite corners of the primitive cell, specified in the lattice basis.) Defaults to unit lengths.
 
   resolution [number or vector3]
       Specifies the computational grid resolution, in pixels per lattice unit (a lattice unit is one basis vector in a given direction).
@@ -53,9 +53,9 @@ class Lattice(object):
       otherwise the resolution is uniform.
       (The grid size is then the product of the lattice size and the resolution, rounded up to the next positive integer.)
       Defaults to 10.
-    
-  If any dimension has the special size no-size, then the dimensionality of the problem is reduced by one; strictly speaking, the dielectric function is taken to be uniform along that dimension. (In this case, the no-size dimension should generally be orthogonal to the other dimensions.) 
-    
+
+  If any dimension has the special size no-size, then the dimensionality of the problem is reduced by one; strictly speaking, the dielectric function is taken to be uniform along that dimension. (In this case, the no-size dimension should generally be orthogonal to the other dimensions.)
+
   .. todo:: This class still needs some work. But works to currently store necessary variables. Use with care.
   .. todo:: merge somehow with BFDTD meshing classes? (need proper definition for resolution, etc)
   '''
@@ -65,14 +65,14 @@ class Lattice(object):
     self.basis3 = array([0,0,1])
     self.basis_size = array([1,1,1])
     self.size = array([1,1,1])
-    
+
     self.xmesh = [0,1]
     self.ymesh = [0,1]
     self.zmesh = [0,1]
     self.setResolution(10, 10, 10)
-    
+
     return
-  
+
   def __str__(self):
     ret = 'lattice:\n'
     ret += '  basis1 = {}\n'.format(self.basis1)
@@ -81,13 +81,13 @@ class Lattice(object):
     ret += '  basis_size = {}\n'.format(self.basis_size)
     ret += '  size = {}'.format(self.size)
     return(ret)
-  
+
   def getLatticeVectors(self):
     a1 = self.size[0]*self.basis_size[0]*self.basis1/norm(self.basis1)
     a2 = self.size[1]*self.basis_size[1]*self.basis2/norm(self.basis2)
     a3 = self.size[2]*self.basis_size[2]*self.basis3/norm(self.basis3)
     return (a1, a2, a3)
-  
+
   def getBounds(self):
     (a1, a2, a3) = self.getLatticeVectors()
 
@@ -101,9 +101,9 @@ class Lattice(object):
     xmax = Pmax[0]
     ymax = Pmax[1]
     zmax = Pmax[2]
-    
+
     return (xmin,xmax, ymin,ymax, zmin,zmax)
-    
+
   def getXmeshDelta(self):
     return(numpy.diff(self.xmesh))
   def getYmeshDelta(self):
@@ -112,7 +112,7 @@ class Lattice(object):
     return(numpy.diff(self.zmesh))
   def getMeshDelta(self):
     return(numpy.diff(self.xmesh),numpy.diff(self.ymesh),numpy.diff(self.zmesh))
-    
+
   def getMinDeltas(self):
     dx = min(self.getXmeshDelta())
     dy = min(self.getYmeshDelta())
@@ -129,7 +129,7 @@ class Lattice(object):
     dy = (ymax-ymin)/(Ny-1)
     dz = (zmax-zmin)/(Nz-1)
     return (dx,dy,dz)
-        
+
   def setResolution(self, Nx, Ny, Nz):
     '''
     Sets up homogeneous X,Y,Z grids with Nx,Ny,Nz points.
@@ -209,7 +209,7 @@ def h5_setupLattice(HDF5_file_object, total_lattice_size=None, requested_dataset
   description = None
   if 'description' in HDF5_file_object.keys():
     #description              Dataset {SCALAR}
-    description = HDF5_file_object['description'][...].tostring().decode("ascii").strip('\0')
+    description = HDF5_file_object['description'][...].tobytes().decode("ascii").strip('\0')
     print('description = {}'.format(description))
 
   complete_dataset_list = h5_getDataSets(HDF5_file_object)
@@ -231,10 +231,10 @@ def h5_setupLattice(HDF5_file_object, total_lattice_size=None, requested_dataset
   if len(data.shape) != 3:
     raise Exception('Data of dimension {}. Only 3D data supported at the moment.'.format(len(data.shape)))
   (Nx, Ny, Nz) = data.shape
-  
+
   # set up lattice
   mylattice = Lattice()
-  
+
   if 'lattice vectors' in HDF5_file_object.keys():
     #lattice\ vectors         Dataset {3, 3}
     lattice_vectors = HDF5_file_object['lattice vectors'][...]
@@ -245,7 +245,7 @@ def h5_setupLattice(HDF5_file_object, total_lattice_size=None, requested_dataset
 
   if total_lattice_size:
     mylattice.setSize(total_lattice_size)
-  
+
   # generate homogeneous mesh by default
   mylattice.setResolution(Nx, Ny, Nz)
 
@@ -255,7 +255,7 @@ def h5_setupLattice(HDF5_file_object, total_lattice_size=None, requested_dataset
     mylattice.setYmesh(HDF5_file_object['ymesh'][...])
   if 'zmesh' in HDF5_file_object.keys():
     mylattice.setZmesh(HDF5_file_object['zmesh'][...])
-  
+
   (a1, a2, a3) = mylattice.getLatticeVectors()
   (xmesh, ymesh, zmesh) = mylattice.getMesh()
   print('a1 = {}'.format(a1))
@@ -266,36 +266,36 @@ def h5_setupLattice(HDF5_file_object, total_lattice_size=None, requested_dataset
   if Nx != len(xmesh) or Ny != len(ymesh) or Nz != len(zmesh):
     raise Exception('Inconsistent number of cells between the dataset and the x/y/z meshs.')
   return (mylattice, complete_dataset_list)
-    
+
 def MPB_h5tovts(h5file, basepath, total_lattice_size=None, requested_dataset=[], verbosity=0):
   '''
   * total_lattice_size : total length of the lattice vectors, i.e. **size*basis-size** in MPB terms. Overrides any values obtained from reading the lattice vectors in the .h5 file.
-  
+
   .. todo:: Emulate the -x/y/z options of mpb-data, i.e. create a periodic structure from a unit-cell.
   '''
   # read in .h5 file
   with h5py.File(h5file, "r") as HDF5_file_object:
     print('Reading from ' + h5file)
-    
+
     (mylattice, complete_dataset_list) = h5_setupLattice(HDF5_file_object, total_lattice_size, requested_dataset)
     (Nx, Ny, Nz) = mylattice.getResolution()
     (a1, a2, a3) = mylattice.getLatticeVectors()
     (xmesh, ymesh, zmesh) = mylattice.getMesh()
-    
+
     # create the vtkPoints structure for the coordinates
     points = vtk.vtkPoints()
     points.SetNumberOfPoints(Nx*Ny*Nz)
-    
+
     if requested_dataset:
       dataset_list = requested_dataset
     else:
       dataset_list = complete_dataset_list
-    
+
     # create the vtkScalarArray structures for the data
     dataset_dict = dict()
     for key in dataset_list:
       print('key = {}'.format(key))
-      
+
       try:
         # We need flat 1D data for VTK structures (and for numpy_to_vtk). Equivalent ways of achieving this:
         #  A.reshape(-1, order='F')
@@ -320,7 +320,7 @@ def MPB_h5tovts(h5file, basepath, total_lattice_size=None, requested_dataset=[],
       dataset_dict[key] = (HDF5_file_object[key], vtk_data, scalar)
 
     last_info_time = time.time()
-    
+
     print('Starting loops')
     counter = 0
     # fill the vtkPoints and vtkScalarArray
@@ -337,24 +337,24 @@ def MPB_h5tovts(h5file, basepath, total_lattice_size=None, requested_dataset=[],
           points.SetPoint(offset, coord)
           #for key in dataset_dict.keys():
             #dataset_dict[key][1].SetTuple1(offset, dataset_dict[key][0][i,j,k])
-          
-          #InsertTuples 	
+
+          #InsertTuples
           #virtual void vtkAbstractArray::InsertTuples 	( 	vtkIdList *  	dstIds,
           #vtkIdList *  	srcIds,
-          #vtkAbstractArray *  	source 
+          #vtkAbstractArray *  	source
           #) 		[pure virtual]
 
           if time.time() - last_info_time > 5:
             print('{} %'.format(100*offset/(Nx*Ny*Nz-1)))
             last_info_time = time.time()
-  
+
           if verbosity>1:
             counter += 1
             progress_str = 'Progress: {}/{}'.format(counter, Nx*Ny*Nz)
             #print(progress_str, end='\r')
             print(progress_str)
             #subprocess.call(["printf", progress_str+'\r'])
-    
+
     print('\nLoops done.')
 
     #for key in dataset_dict.keys():
@@ -362,24 +362,24 @@ def MPB_h5tovts(h5file, basepath, total_lattice_size=None, requested_dataset=[],
       #vtk_data = dataset_dict[key][1]
       #vtk_data.InsertTuples()
       #.SetTuple1(offset, [i,j,k])
-    
+
     # create structured grid
     dataset_vts = vtk.vtkStructuredGrid()
     dataset_vts.SetDimensions(Nx, Ny, Nz)
     dataset_vts.SetPoints(points)
-    
+
     # create vtkImageData
     dataset_vti = vtk.vtkImageData()
     dataset_vti.SetDimensions(Nx, Ny, Nz)
     (xmin,xmax, ymin,ymax, zmin,zmax) = mylattice.getBounds()
     dataset_vti.SetOrigin([xmin, ymin, zmin])
     dataset_vti.SetSpacing(mylattice.getSpacing())
-    
+
     # add scalar data to the grids
     for key in dataset_dict.keys():
       dataset_vts.GetPointData().AddArray(dataset_dict[key][1])
       dataset_vti.GetPointData().AddArray(dataset_dict[key][1])
-    
+
     dataset_vts.GetPointData().SetActiveScalars('data')
     dataset_vti.GetPointData().SetActiveScalars('data')
 
@@ -403,7 +403,7 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
     .. todo:: need py2-compatible bfdtd-parser for direct creation
     .. todo:: when selecting a smaller mesh region for speed, we also need to change the size of the VTS grid (but keep coords correct)...
   '''
-  
+
   requested_arrays = ['material', 'E', 'electric_energy_density']
   #requested_arrays = []
   #x_relative_range = [1/3, 2/3]
@@ -412,19 +412,19 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
   print('x_relative_range: {}'.format(x_relative_range))
   print('y_relative_range: {}'.format(y_relative_range))
   print('z_relative_range: {}'.format(z_relative_range))
-  
+
   print('{} -> {}.vts'.format(h5file, basepath))
-  
+
   # read in .h5 file
   with h5py.File(h5file, "r") as HDF5_file_object:
     print('Reading from ' + h5file)
-    
+
     (mylattice, complete_dataset_list) = h5_setupLattice(HDF5_file_object, total_lattice_size, requested_dataset)
     (Nx, Ny, Nz) = mylattice.getResolution()
     (a1, a2, a3) = mylattice.getLatticeVectors()
     (xmesh, ymesh, zmesh) = mylattice.getMesh()
     Npoints = Nx*Ny*Nz
-    
+
     # create the vtkPoints structure for the coordinates
     #points = vtk.vtkPoints()
     #points.SetNumberOfPoints(Npoints)
@@ -440,7 +440,7 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
       if 'material' in current_data.dtype.names:
         material_data = current_data
         break
-        
+
     if material_data:
       print('Found material data in: {}'.format(material_data.name))
     else:
@@ -452,7 +452,7 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
       # create VTK data arrays
       print('Creating VTK data arrays...')
       available_arrays = []
-      
+
       if data.dtype.names == ('E', 'H', 'Pow', 'material'):
         print('time snapshot mode')
         if 'E' in requested_arrays:
@@ -468,7 +468,7 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
           material_array = createvtkArray('material', Npoints, ('material',))
           available_arrays.append(material_array)
         #available_arrays = (E_array, H_array, Pow_array, material_array)
-      
+
       elif data.dtype.names == ('E', 'H'):
         print('frequency snapshot mode')
         if 'E' in requested_arrays:
@@ -495,9 +495,9 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
         raise Exception('Unsupported data type.')
 
       print('...done')
-            
+
       last_info_time = time.time()
-      
+
       x_range = range( int(round(x_relative_range[0]*Nx)), int(round(x_relative_range[1]*Nx)) )
       y_range = range( int(round(y_relative_range[0]*Ny)), int(round(y_relative_range[1]*Ny)) )
       z_range = range( int(round(z_relative_range[0]*Nz)), int(round(z_relative_range[1]*Nz)) )
@@ -505,16 +505,16 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
       Ny_partial = len(y_range)
       Nz_partial = len(z_range)
       Npoints_partial = Nx_partial*Ny_partial*Nz_partial
-      
+
       print( 'Range to fill:' )
       print( 'x: {} -> size={}/{}'.format((x_range[0], x_range[-1]), Nx_partial, Nx) )
       print( 'y: {} -> size={}/{}'.format((y_range[0], y_range[-1]), Ny_partial, Ny) )
       print( 'z: {} -> size={}/{}'.format((z_range[0], z_range[-1]), Nz_partial, Nz) )
       print( 'total points to fill: {}/{}'.format(Npoints_partial, Npoints) )
-      
+
       if dry_run:
         return
-      
+
       print('Starting loops')
       # fill the vtkPoints and vtkScalarArray
       for k in z_range: #range(Nz):
@@ -523,7 +523,7 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
       #for k in range(Nz):
         #for j in range(Ny):
           #for i in range(Nx):
-            
+
             offset = i + j*Nx + k*Nx*Ny
             #coord = xmesh[i]*a1 + ymesh[j]*a2 + zmesh[k]*a3
             #points.SetPoint(offset, coord)
@@ -544,10 +544,10 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
                 E_array.SetTuple3(offset, *E)
               if 'H' in requested_arrays:
                 H_array.SetTuple3(offset, *H)
-            
+
             elif data.dtype.names == ('E', 'H'):
               S = numpy.cross(E, numpy.conj(H))
-              
+
               if 'E' in requested_arrays:
                 E_mod = setTupleComplex(offset, E, E_re_array, E_im_array, E_mod_array, E_phase_array)
               if 'H' in requested_arrays:
@@ -574,15 +574,15 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
 
             else:
               raise Exception('Unsupported data type.')
-            
+
             if time.time() - last_info_time > 5 or verbosity > 1:
               #print('Progress: {}/{} = {} %'.format(offset+1, Npoints, 100*(offset+1)/Npoints))
               offset_partial = (i-x_range[0]) + (j-y_range[0])*Nx_partial + (k-z_range[0])*Nx_partial*Ny_partial
               print('Progress: {}/{} = {} %'.format(offset_partial+1, Npoints_partial, 100*(offset_partial+1)/Npoints_partial))
               last_info_time = time.time()
-      
+
       print('Loops done.')
-      
+
       # create structured grid
       #dataset_vts = vtk.vtkStructuredGrid()
       #dataset_vts.SetDimensions(Nx, Ny, Nz)
@@ -592,11 +592,11 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
       xCoords = vtkScalarArray()
       for i in range(len(xmesh)):
         xCoords.InsertNextValue(xmesh[i])
-      
+
       yCoords = vtkScalarArray()
       for i in range(len(ymesh)):
         yCoords.InsertNextValue(ymesh[i])
-      
+
       zCoords = vtkScalarArray()
       for i in range(len(zmesh)):
         zCoords.InsertNextValue(zmesh[i])
@@ -606,10 +606,10 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
       dataset_vtr.SetXCoordinates(xCoords)
       dataset_vtr.SetYCoordinates(yCoords)
       dataset_vtr.SetZCoordinates(zCoords)
-      
+
       for A in available_arrays:
         dataset_vtr.GetPointData().AddArray(A)
-      
+
       # write out VTK file(s)
       basepath_snapshot = basepath + data.name.replace('/','_')
 
@@ -617,7 +617,7 @@ def BFDTD_h5_to_vts(h5file, basepath, total_lattice_size=None, requested_dataset
       #writer_vts.SetFileName(basepath_snapshot + '.' + writer_vts.GetDefaultFileExtension())
       #writer_vts.SetInputData(dataset_vts)
       #writer_vts.Write()
-      
+
       writer_vtr = vtk.vtkXMLRectilinearGridWriter()
       writer_vtr.SetFileName(basepath_snapshot + '.' + writer_vtr.GetDefaultFileExtension())
       writer_vtr.SetInputData(dataset_vtr)
@@ -645,7 +645,7 @@ def createvtkArray(name, number_of_tuples, component_names):
     A.SetComponentName(idx, s)
     A.FillComponent(idx, 0)
   return A
-  
+
 def createComplexArrays(name, number_of_tuples):
   #A_re = createvtkArray(name+'_re', number_of_tuples, (name+'_re_x', name+'_re_y', name+'_re_z'))
   #A_im = createvtkArray(name+'_im', number_of_tuples, (name+'_im_x', name+'_im_y', name+'_im_z'))
@@ -658,13 +658,13 @@ def createComplexArrays(name, number_of_tuples):
   return (A_re, A_im, A_mod, A_phase)
 
 def stltoh5(stlfile, basepath, epsilon_inside, epsilon_outside, lattice=Lattice(), verbosity=0):
-  
+
   if not os.path.exists(stlfile):
     if sys.version_info.major == 2:
       raise IOError('No such file or directory: {}'.format(stlfile)) # py2
     else:
       raise FileNotFoundError('No such file or directory: {}'.format(stlfile)) # py3
-  
+
   print('--> timer start')
   time_start = time.time()
 
@@ -685,7 +685,7 @@ def stltoh5(stlfile, basepath, epsilon_inside, epsilon_outside, lattice=Lattice(
   implicit_function.SetInput(polydata)
 
   print("--> Elapsed time: %.4f sec" % (time.time() - time_start))
-  
+
   stl_to_vts_and_h5(implicit_function, basepath, lattice, epsilon_inside, epsilon_outside)
   print("--> Elapsed time: %.4f sec" % (time.time() - time_start))
   return
@@ -693,7 +693,7 @@ def stltoh5(stlfile, basepath, epsilon_inside, epsilon_outside, lattice=Lattice(
 def stl_to_vts_and_h5(implicit_function, outfile_basename, lattice, epsilon_inside, epsilon_outside):
   (Nx, Ny, Nz) = lattice.getResolution()
   (a1, a2, a3) = lattice.getLatticeVectors()
-  
+
   points = vtk.vtkPoints()
   points.SetNumberOfPoints(Nx*Ny*Nz)
 
@@ -703,7 +703,7 @@ def stl_to_vts_and_h5(implicit_function, outfile_basename, lattice, epsilon_insi
 
   print('=== dims ===')
   print(scalars_numpy.shape)
-  
+
   last_info_time = time.time()
 
   print('=== Loop start ===')
@@ -719,11 +719,11 @@ def stl_to_vts_and_h5(implicit_function, outfile_basename, lattice, epsilon_insi
           value = epsilon_outside
         scalars_vtk.SetTuple1(offset, value)
         scalars_numpy[i, j, k] = value
-        
+
         if time.time() - last_info_time > 5:
           print('{} %'.format(100*offset/(Nx*Ny*Nz-1)))
           last_info_time = time.time()
-  
+
   print('=== Loop end ===')
 
   dataset = vtk.vtkStructuredGrid()
@@ -735,23 +735,23 @@ def stl_to_vts_and_h5(implicit_function, outfile_basename, lattice, epsilon_insi
   writer.SetInputData(dataset)
   writer.SetFileName(outfile_basename + '.' + writer.GetDefaultFileExtension())
   writer.Write()
-  
+
   h5file = outfile_basename + '.h5'
   with h5py.File(h5file, "w") as HDF5_file_object:
     print('writing to ' + h5file)
-    
+
     dset = HDF5_file_object.create_dataset('/data', scalars_numpy.shape, dtype=numpy.float64)
     dset[...] = scalars_numpy
 
     dset = HDF5_file_object.create_dataset("description", (), dtype="S29")
     dset[...] = 'dielectric function, epsilon'
-    
+
     lattice_vectors = numpy.array([a1, a2, a3])
     print(lattice_vectors)
 
     dset = HDF5_file_object.create_dataset('/lattice vectors', lattice_vectors.shape, dtype=numpy.float64)
     dset[...] = lattice_vectors
-    
+
     # TODO: Add these fields:
     #epsilon.xx               Dataset {100, 100, 100}
     #epsilon.xy               Dataset {100, 100, 100}
@@ -765,7 +765,7 @@ def stl_to_vts_and_h5(implicit_function, outfile_basename, lattice, epsilon_insi
     #epsilon_inverse.yy       Dataset {100, 100, 100}
     #epsilon_inverse.yz       Dataset {100, 100, 100}
     #epsilon_inverse.zz       Dataset {100, 100, 100}
-  
+
   return
 
 if __name__ == '__main__':
